@@ -193,8 +193,8 @@ def hooks_to_env(env, arg_hooks, hook):
            help=_('Name of the stack to adopt.'))
 def do_stack_adopt(hc, args):
     '''Adopt a stack.'''
-    env_files, env = template_utils.process_environment_and_files(
-        env_path=args.environment_file)
+    env_files, env = template_utils.process_multiple_environments_and_files(
+        env_paths=args.environment_file)
 
     if not args.adopt_file:
         raise exc.CommandError(_('Need to specify %(arg)s') %
@@ -428,6 +428,9 @@ def do_stack_show(hc, args):
            'Values %(false)s set rollback to disabled. '
            'Default is to use the value of existing stack to be updated.')
            % {'true': strutils.TRUE_STRINGS, 'false': strutils.FALSE_STRINGS})
+@utils.arg('-y', '--dry-run', default=False, action="store_true",
+           help='Do not actually perform the stack update, but show what '
+           'would be changed')
 @utils.arg('-P', '--parameters', metavar='<KEY1=VALUE1;KEY2=VALUE2...>',
            help=_('Parameter values used to create the stack. '
            'This can be specified multiple times, or once with parameters '
@@ -439,11 +442,14 @@ def do_stack_show(hc, args):
            'would be the content of the file'),
            action='append')
 @utils.arg('-x', '--existing', default=False, action="store_true",
-           help=_('Re-use the set of parameters of the current stack. '
+           help=_('Re-use the template, parameters and environment of the '
+           'current stack. If the template argument is omitted then the '
+           'existing template is used. If no %(env_arg)s is specified then '
+           'the existing environment is used. '
            'Parameters specified in %(arg)s will patch over the existing '
            'values in the current stack. Parameters omitted will keep '
            'the existing values.')
-           % {'arg': '--parameters'})
+           % {'arg': '--parameters', 'env_arg': '--environment-file'})
 @utils.arg('-c', '--clear-parameter', metavar='<PARAMETER>',
            help=_('Remove the parameters from the set of parameters of '
            'current stack for the %(cmd)s. The default value in the '
@@ -461,7 +467,8 @@ def do_stack_update(hc, args):
         args.template_file,
         args.template_url,
         args.template_object,
-        _authenticated_fetcher(hc))
+        _authenticated_fetcher(hc),
+        existing=args.existing)
 
     env_files, env = template_utils.process_multiple_environments_and_files(
         env_paths=args.environment_file)
@@ -501,6 +508,27 @@ def do_stack_update(hc, args):
         if args.enable_rollback:
             fields['disable_rollback'] = False
 
+    if args.dry_run is True:
+        resource_changes = hc.stacks.preview_update(**fields)
+
+        formatters = {
+            'resource_identity': utils.json_formatter
+        }
+
+        fields = ['state', 'resource_name', 'resource_type',
+                  'resource_identity']
+
+        for k in resource_changes.get("resource_changes", {}).keys():
+            for i in range(len(resource_changes["resource_changes"][k])):
+                resource_changes["resource_changes"][k][i]['state'] = k
+
+        utils.print_update_list(
+            sum(resource_changes["resource_changes"].values(), []),
+            fields,
+            formatters=formatters
+        )
+        return
+
     hc.stacks.update(**fields)
     do_stack_list(hc)
 
@@ -545,6 +573,16 @@ def do_stack_cancel_update(hc, args):
            help=_('Limit the number of stacks returned.'))
 @utils.arg('-m', '--marker', metavar='<ID>',
            help=_('Only return stacks that appear after the given stack ID.'))
+@utils.arg('-k', '--sort-keys', metavar='<KEY1;KEY2...>',
+           help=_('List of keys for sorting the returned stacks. '
+                  'This can be specified multiple times or once with keys '
+                  'separated by semicolons. Valid sorting keys include '
+                  '"stack_name", "stack_status", "creation_time" and '
+                  '"updated_time".'),
+           action='append')
+@utils.arg('-d', '--sort-dir', metavar='[asc|desc]',
+           help=_('Sorting direction (either "asc" or "desc") for the sorting '
+                  'keys.'))
 @utils.arg('-g', '--global-tenant', action='store_true', default=False,
            help=_('Display stacks from all tenants. Operation only authorized '
                   'for users who match the policy in heat\'s policy.json.'))
@@ -554,7 +592,11 @@ def do_stack_cancel_update(hc, args):
 def do_stack_list(hc, args=None):
     '''List the user's stacks.'''
     kwargs = {}
-    fields = ['id', 'stack_name', 'stack_status', 'creation_time']
+    fields = ['id', 'stack_name', 'stack_status', 'creation_time',
+              'updated_time']
+    sort_keys = ['stack_name', 'stack_status', 'creation_time',
+                 'updated_time']
+    sortby_index = 3
     if args:
         kwargs = {'limit': args.limit,
                   'marker': args.marker,
@@ -570,13 +612,36 @@ def do_stack_list(hc, args=None):
             fields.append('parent')
             kwargs['show_nested'] = True
 
+        if args.sort_keys:
+            # flatten key list first
+            keys = []
+            for k in args.sort_keys:
+                if ';' in k:
+                    keys.extend(k.split(';'))
+                else:
+                    keys.append(k)
+            # validate key list
+            for key in keys:
+                if key not in sort_keys:
+                    err = _("Sorting key '%(key)s' not one of the supported "
+                            "keys: %(keys)s") % {'key': key, "keys": sort_keys}
+                    raise exc.CommandError(err)
+            kwargs['sort_keys'] = keys
+            sortby_index = None
+
+        if args.sort_dir:
+            if args.sort_dir not in ('asc', 'desc'):
+                raise exc.CommandError(_("Sorting direction must be one of "
+                                         "'asc' and 'desc'"))
+            kwargs['sort_dir'] = args.sort_dir
+
         if args.global_tenant or args.show_owner:
             fields.insert(2, 'stack_owner')
         if args.global_tenant:
             fields.insert(2, 'project')
 
     stacks = hc.stacks.list(**kwargs)
-    utils.print_list(stacks, fields, sortby_index=3)
+    utils.print_list(stacks, fields, sortby_index=sortby_index)
 
 
 @utils.arg('id', metavar='<NAME or ID>',
@@ -641,9 +706,16 @@ def do_output_show(hc, args):
                 print(value)
 
 
+@utils.arg('-f', '--filters', metavar='<KEY1=VALUE1;KEY2=VALUE2...>',
+           help=_('Filter parameters to apply on returned resource types. '
+           'This can be specified multiple times, or once with parameters '
+           'separated by a semicolon. It can be any of name, version and '
+           'support_status'),
+           action='append')
 def do_resource_type_list(hc, args):
     '''List the available resource types.'''
-    types = hc.resource_types.list()
+    types = hc.resource_types.list(
+        filters=utils.format_parameters(args.filters))
     utils.print_list(types, ['resource_type'], sortby_index=0)
 
 
@@ -710,6 +782,8 @@ def do_template_show(hc, args):
            action='append')
 @utils.arg('-o', '--template-object', metavar='<URL>',
            help=_('URL to retrieve template object (e.g. from swift).'))
+@utils.arg('-n', '--show-nested', default=False, action="store_true",
+           help=_('Resolve parameters from nested templates as well.'))
 def do_template_validate(hc, args):
     '''Validate a template with parameters.'''
 
@@ -724,9 +798,12 @@ def do_template_validate(hc, args):
     fields = {
         'template': template,
         'files': dict(list(tpl_files.items()) + list(env_files.items())),
-        'environment': env
+        'environment': env,
     }
 
+    if args.show_nested:
+        fields['show_nested'] = args.show_nested
+
     validation = hc.stacks.validate(**fields)
     print(jsonutils.dumps(validation, indent=2, ensure_ascii=False))
 
